@@ -21,10 +21,7 @@ import datetime
 from email import utils
 import errno
 import os
-import pwd
-import re
 import signal
-import socket
 import shutil
 import subprocess
 import tempfile
@@ -36,10 +33,18 @@ except ImportError:
     # debian_bundle
     from debian_bundle import changelog, deb822
 
+try:
+    get_maintainer = changelog.get_maintainer
+except AttributeError:
+    # Implementation of get_maintainer was added after 0.1.18 so import same
+    # function from backports module if python-debian doesn't have it.
+    from bzrlib.plugins.builder.backports import get_maintainer
+
 from bzrlib import (
         errors,
         export as _mod_export,
         lazy_regex,
+        osutils,
         trace,
         transport as _mod_transport,
         urlutils,
@@ -50,7 +55,6 @@ from bzrlib.option import Option
 
 from bzrlib.plugins.builder.recipe import (
         BaseRecipeBranch,
-        DebUpstreamVariable,
         build_tree,
         RecipeParser,
         resolve_revisions,
@@ -131,71 +135,6 @@ def get_old_recipe(if_changed_from, possible_transports=None):
     return old_recipe
 
 
-def get_maintainer():
-    """Create maintainer string using the same algorithm as in dch.
-    """
-    env = os.environ
-    regex = re.compile(r"^(.*)\s+<(.*)>$")
-
-    # Split email and name
-    if 'DEBEMAIL' in env:
-        match_obj = regex.match(env['DEBEMAIL'])
-        if match_obj:
-            if not 'DEBFULLNAME' in env:
-                env['DEBFULLNAME'] = match_obj.group(1)
-            env['DEBEMAIL'] = match_obj.group(2)
-    if 'DEBEMAIL' not in env or 'DEBFULLNAME' not in env:
-        if 'EMAIL' in env:
-            match_obj = regex.match(env['EMAIL'])
-            if match_obj:
-                if not 'DEBFULLNAME' in env:
-                    env['DEBFULLNAME'] = match_obj.group(1)
-                env['EMAIL'] = match_obj.group(2)
-
-    # Get maintainer's name
-    if 'DEBFULLNAME' in env:
-        maintainer = env['DEBFULLNAME']
-    elif 'NAME' in env:
-        maintainer = env['NAME']
-    else:
-        # Use password database if no data in environment variables
-        try:
-            maintainer = re.sub(r',.*', '', pwd.getpwuid(os.getuid()).pw_gecos)
-        except (KeyError, AttributeError):
-            # TBD: Use last changelog entry value
-            maintainer = "bzr-builder"
-
-    # Get maintainer's mail address
-    if 'DEBEMAIL' in env:
-        email = env['DEBEMAIL']
-    elif 'EMAIL' in env:
-        email = env['EMAIL']
-    else:
-        addr = None
-        if os.path.exists('/etc/mailname'):
-            f = open('/etc/mailname')
-            try:
-                addr = f.readline().strip()
-            finally:
-                f.close()
-        if not addr:
-            addr = socket.getfqdn()
-        if addr:
-            user = pwd.getpwuid(os.getuid()).pw_name
-            if not user:
-                addr = None
-            else:
-                addr = "%s@%s" % (user, addr)
-
-        if addr:
-            email = addr
-        else:
-            # TBD: Use last changelog entry value
-            email = "none@example.org"
-
-    return (maintainer, email)
-
-
 def add_autobuild_changelog_entry(base_branch, basedir, package,
         distribution=None, author_name=None, author_email=None,
         append_version=None):
@@ -242,14 +181,21 @@ def add_autobuild_changelog_entry(base_branch, basedir, package,
     if base_branch.format in (0.1, 0.2, 0.3):
         try:
             base_branch.substitute_changelog_vars(None, cl)
-        except SubstitutionUnavailable:
+        except SubstitutionUnavailable, e:
             raise errors.BzrCommandError("No previous changelog to "
                     "take the upstream version from as %s was "
-                    "used: %s." % (DebUpstreamVariable.name, reason))
+                    "used: %s: %s." % (e.name, e.reason, reason))
     # Use debian packaging environment variables
     # or default values if they don't exist
     if author_name is None or author_email is None:
         author_name, author_email = get_maintainer()
+        # The python-debian package breaks compatibility at version 0.1.20 by
+        # switching to expecting (but not checking for) unicode rather than
+        # bytestring inputs. Detect this and decode environment if needed.
+        if getattr(changelog.Changelog, "__unicode__", None) is not None:
+            enc = osutils.get_user_encoding()
+            author_name = author_name.decode(enc)
+            author_email = author_email.decode(enc)
     author = "%s <%s>" % (author_name, author_email)
 
     date = utils.formatdate(localtime=True)
@@ -386,12 +332,11 @@ def convert_3_0_quilt_to_native(path):
         f.close()
 
 
-def force_native_format(working_tree_path):
+def force_native_format(working_tree_path, current_format):
     """Make sure a package is a format that supports native packages.
 
     :param working_tree_path: Path to the package
     """
-    current_format = get_source_format(working_tree_path)
     if current_format == "3.0 (quilt)":
         convert_3_0_quilt_to_native(working_tree_path)
     elif current_format not in ("1.0", "3.0 (native)"):
@@ -545,9 +490,13 @@ def debian_source_package_name(control_path):
     """Open a debian control file and extract the package name.
 
     """
-    with open(control_path, 'r') as f:
+    f = open(control_path, 'r')
+    try:
         control = deb822.Deb822(f)
-        return control["Source"]
+        # Debian policy states package names are [a-z0-9][a-z0-9.+-]+ so ascii
+        return control["Source"].encode("ascii")
+    finally:
+        f.close()
 
 
 def reconstruct_pristine_tar(dest, delta, dest_filename):
@@ -727,16 +676,21 @@ class cmd_dailydeb(cmd_build):
                     write_manifest_to_transport(manifest, base_branch,
                         possible_transports)
                 return 0
-            if package_version.debian_version is not None:
+            current_format = get_source_format(package_dir)
+            if (package_version.debian_version is not None or
+                current_format == "3.0 (quilt)"):
                 # Non-native package
                 try:
                     extract_upstream_tarball(base_branch.branch, package_name,
                         package_version.upstream_version, working_basedir)
-                except errors.NoSuchTag:
+                except errors.NoSuchTag, e:
                     if not allow_fallback_to_native:
-                        raise
-            if allow_fallback_to_native:
-                force_native_format(package_dir)
+                        raise errors.BzrCommandError(
+                            "Unable to find the upstream source. Import it "
+                            "as tag %s or build with "
+                            "--allow-fallback-to-native." % e.tag_name)
+                    else:
+                        force_native_format(package_dir, current_format)
             try:
                 build_source_package(package_dir,
                         tgz_check=not allow_fallback_to_native)
