@@ -27,7 +27,9 @@ from bzrlib import (
 from bzrlib.branch import Branch
 from bzrlib.tests import (
         TestCaseWithTransport,
+        TestSkipped,
         )
+from bzrlib.tests.features import Feature
 
 
 from bzrlib.plugins.builder.tests import PristineTarFeature
@@ -38,6 +40,17 @@ except ImportError:
     from debian_bundle import changelog
 
 
+class _NotUnderFakeRootFeature(Feature):
+
+    def _probe(self):
+        return 'FAKEROOTKEY' not in os.environ
+
+    def feature_name(self):
+        return 'not running inside fakeroot'
+
+NotUnderFakeRootFeature = _NotUnderFakeRootFeature()
+
+
 def make_pristine_tar_delta(dest, tarball_path):
     """Create a pristine-tar delta for a tarball.
 
@@ -82,6 +95,17 @@ class BlackboxBuilderTests(TestCaseWithTransport):
         overrideEnv("DEBEMAIL", "maint@maint.org")
         overrideEnv("DEBFULLNAME", "M. Maintainer")
 
+    def _get_file_contents(self, filename, mode="r"):
+        """Helper to read contents of a file
+
+        Use check_file_content instead to just assert the contents match."""
+        self.assertPathExists(filename)
+        f = open(filename, mode)
+        try:
+            return f.read()
+        finally:
+            f.close()
+
     def test_cmd_builder_exists(self):
         self.run_bzr("build --help")
 
@@ -201,13 +225,15 @@ class BlackboxBuilderTests(TestCaseWithTransport):
                     "deb-version 1\nsource revid:%s\n" % revid)
 
     def test_cmd_dailydeb(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         #TODO: define a test feature for debuild and require it here.
         source = self.make_branch_and_tree("source")
         self.build_tree(["source/a", "source/debian/"])
         self.build_tree_contents([("source/debian/rules",
                     "#!/usr/bin/make -f\nclean:\n"),
                 ("source/debian/control",
-                    "Source: foo\nMaintainer: maint maint@maint.org\n")])
+                    "Source: foo\nMaintainer: maint maint@maint.org\n\n"
+                    "Package: foo\nArchitecture: all\n")])
         source.add(["a", "debian/", "debian/rules", "debian/control"])
         revid = source.commit("one")
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.1 "
@@ -226,16 +252,13 @@ class BlackboxBuilderTests(TestCaseWithTransport):
                     "debian/bzr-builder.manifest"),
                     "# bzr-builder format 0.1 deb-version 1\nsource revid:%s\n"
                     % revid)
-        cl_path = os.path.join(package_root, "debian/changelog")
-        self.assertPathExists(cl_path)
-        cl_f = open(cl_path)
-        try:
-            line = cl_f.readline()
-            self.assertEqual("foo (1) lucid; urgency=low\n", line)
-        finally:
-            cl_f.close()
+        cl_contents = self._get_file_contents(
+            os.path.join(package_root, "debian/changelog"))
+        self.assertEqual("foo (1) lucid; urgency=low\n",
+            cl_contents.splitlines(True)[0])
 
     def test_cmd_dailydeb_no_work_dir(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         #TODO: define a test feature for debuild and require it here.
         if getattr(self, "permit_dir", None) is not None:
             self.permit_dir('/') # Allow the made working dir to be accessed.
@@ -244,7 +267,8 @@ class BlackboxBuilderTests(TestCaseWithTransport):
         self.build_tree_contents([("source/debian/rules",
                     "#!/usr/bin/make -f\nclean:\n"),
                 ("source/debian/control",
-                    "Source: foo\nMaintainer: maint maint@maint.org\n")])
+                    "Source: foo\nMaintainer: maint maint@maint.org\n\n"
+                    "Package: foo\nArchitecture: all\n")])
         source.add(["a", "debian/", "debian/rules", "debian/control"])
         source.commit("one")
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.1 "
@@ -253,6 +277,7 @@ class BlackboxBuilderTests(TestCaseWithTransport):
                 "--manifest manifest")
 
     def test_cmd_dailydeb_if_changed_from_non_existant(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         #TODO: define a test feature for debuild and require it here.
         if getattr(self, "permit_dir", None) is not None:
             self.permit_dir('/') # Allow the made working dir to be accessed.
@@ -261,7 +286,8 @@ class BlackboxBuilderTests(TestCaseWithTransport):
         self.build_tree_contents([("source/debian/rules",
                     "#!/usr/bin/make -f\nclean:\n"),
                 ("source/debian/control",
-                    "Source: foo\nMaintainer: maint maint@maint.org\n")])
+                    "Source: foo\nMaintainer: maint maint@maint.org\n"
+                    "\nPackage: foo\nArchitecture: all\n")])
         source.add(["a", "debian/", "debian/rules", "debian/control"])
         source.commit("one")
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.1 "
@@ -314,7 +340,8 @@ class BlackboxBuilderTests(TestCaseWithTransport):
             (os.path.join(path, "debian/rules"),
                 "#!/usr/bin/make -f\nclean:\n"),
             (os.path.join(path, "debian/control"),
-                "Source: package\nMaintainer: maint maint@maint.org\n"),
+                "Source: package\nMaintainer: maint maint@maint.org\n\n"
+                "Package: package\nArchitecture: all\n"),
             (os.path.join(path, "debian/changelog"),
                 cl_contents)
             ])
@@ -331,17 +358,15 @@ class BlackboxBuilderTests(TestCaseWithTransport):
                 "--manifest manifest --no-build working")
         new_cl_contents = ("package (1) unstable; urgency=low\n\n"
                 "  * Auto build.\n\n -- M. Maintainer <maint@maint.org>  ")
-        f = open("working/package-1/debian/changelog")
-        try:
-            actual_cl_contents = f.read()
-        finally:
-            f.close()
+        actual_cl_contents = self._get_file_contents(
+            "working/package-1/debian/changelog")
         self.assertStartsWith(actual_cl_contents, new_cl_contents)
         for fn in os.listdir("working"):
             self.assertFalse(fn.endswith(".changes"))
 
     def test_cmd_dailydeb_with_package_from_changelog(self):
         #TODO: define a test feature for debuild and require it here.
+        self.requireFeature(NotUnderFakeRootFeature)
         self.make_simple_package("source")
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.1 "
                     "deb-version 1\nsource 1\n")])
@@ -349,14 +374,12 @@ class BlackboxBuilderTests(TestCaseWithTransport):
                 "--manifest manifest --if-changed-from bar working")
         new_cl_contents = ("package (1) unstable; urgency=low\n\n"
                 "  * Auto build.\n\n -- M. Maintainer <maint@maint.org>  ")
-        f = open("working/test-1/debian/changelog")
-        try:
-            actual_cl_contents = f.read()
-        finally:
-            f.close()
+        actual_cl_contents = self._get_file_contents(
+            "working/test-1/debian/changelog")
         self.assertStartsWith(actual_cl_contents, new_cl_contents)
 
     def test_cmd_dailydeb_with_version_from_changelog(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         self.make_simple_package("source")
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.1 "
                     "deb-version {debversion}-2\nsource 1\n")])
@@ -364,15 +387,12 @@ class BlackboxBuilderTests(TestCaseWithTransport):
             "dailydeb --allow-fallback-to-native -q test.recipe working")
         new_cl_contents = ("package (0.1-2) unstable; urgency=low\n\n"
                 "  * Auto build.\n\n -- M. Maintainer <maint@maint.org>  ")
-        f = open("working/test-{debversion}-2/debian/changelog")
-        try:
-            actual_cl_contents = f.read()
-        finally:
-            f.close()
-        cl = changelog.Changelog(actual_cl_contents)
+        cl = changelog.Changelog(self._get_file_contents(
+            "working/test-{debversion}-2/debian/changelog"))
         self.assertEquals("0.1-1-2", str(cl._blocks[0].version))
 
     def test_cmd_dailydeb_with_version_from_other_branch_changelog(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         source = self.make_simple_package("source")
         other = self.make_simple_package("other")
         cl_contents = ("package (0.4-1) unstable; urgency=low\n  * foo\n"
@@ -388,15 +408,12 @@ class BlackboxBuilderTests(TestCaseWithTransport):
             "nest other other other\n")])
         out, err = self.run_bzr(
             "dailydeb --allow-fallback-to-native -q test.recipe working")
-        f = open("working/test-{debversion:other}.2/debian/changelog")
-        try:
-            actual_cl_contents = f.read()
-        finally:
-            f.close()
-        cl = changelog.Changelog(actual_cl_contents)
+        cl = changelog.Changelog(self._get_file_contents(
+            "working/test-{debversion:other}.2/debian/changelog"))
         self.assertEquals("0.4-1.2", str(cl._blocks[0].version))
 
     def test_cmd_dailydeb_with_upstream_version_from_changelog(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         self.make_simple_package("source")
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.1 "
                     "deb-version {debupstream}-2\nsource 1\n")])
@@ -404,14 +421,12 @@ class BlackboxBuilderTests(TestCaseWithTransport):
             "dailydeb --allow-fallback-to-native -q test.recipe working")
         new_cl_contents = ("package (0.1-2) unstable; urgency=low\n\n"
                 "  * Auto build.\n\n -- M. Maintainer <maint@maint.org>  ")
-        f = open("working/test-{debupstream}-2/debian/changelog")
-        try:
-            actual_cl_contents = f.read()
-        finally:
-            f.close()
+        actual_cl_contents = self._get_file_contents(
+            "working/test-{debupstream}-2/debian/changelog")
         self.assertStartsWith(actual_cl_contents, new_cl_contents)
 
     def test_cmd_dailydeb_with_append_version(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         self.make_simple_package("source")
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.1 "
                     "deb-version 1\nsource 1\n")])
@@ -419,11 +434,26 @@ class BlackboxBuilderTests(TestCaseWithTransport):
                 "--append-version ~ppa1")
         new_cl_contents = ("package (1~ppa1) unstable; urgency=low\n\n"
                 "  * Auto build.\n\n -- M. Maintainer <maint@maint.org>  ")
-        f = open("working/test-1/debian/changelog")
+        actual_cl_contents = self._get_file_contents(
+            "working/test-1/debian/changelog")
+        self.assertStartsWith(actual_cl_contents, new_cl_contents)
+
+    def test_cmd_dailydeb_with_nonascii_maintainer_in_changelog(self):
+        user_enc = osutils.get_user_encoding()
         try:
-            actual_cl_contents = f.read()
-        finally:
-            f.close()
+            os.environ["DEBFULLNAME"] = u"Micha\u25c8 Sawicz".encode(user_enc)
+        except UnicodeEncodeError:
+            self.skip("Need user encoding other than %r to test maintainer "
+                "name from environment" % (user_enc,))
+        self.make_simple_package("source")
+        self.build_tree_contents([("test.recipe", "# bzr-builder format 0.1 "
+                    "deb-version 1\nsource 1\n")])
+        out, err = self.run_bzr("dailydeb -q test.recipe working")
+        new_cl_contents = ("package (1) unstable; urgency=low\n\n"
+            "  * Auto build.\n\n"
+            " -- Micha\xe2\x97\x88 Sawicz <maint@maint.org>  ")
+        actual_cl_contents = self._get_file_contents(
+            "working/test-1/debian/changelog")
         self.assertStartsWith(actual_cl_contents, new_cl_contents)
 
     def test_cmd_dailydeb_with_invalid_version(self):
@@ -468,9 +498,13 @@ class BlackboxBuilderTests(TestCaseWithTransport):
             "dailydeb -q test.recipe working",
             retcode=3)
         self.assertEquals("", out)
-        self.assertEquals('bzr: ERROR: No such tag: upstream-1\n', err)
+        self.assertEquals(
+            'bzr: ERROR: Unable to find the upstream source. '
+            'Import it as tag upstream-1 or build with '
+            '--allow-fallback-to-native.\n', err)
 
     def test_cmd_dailydeb_with_orig_tarball(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         self.make_simple_package("source")
         self.make_upstream_version("0.1", [("upstream/file", "content\n")])
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.3 "
@@ -482,6 +516,7 @@ class BlackboxBuilderTests(TestCaseWithTransport):
         self.assertPathExists("working/package_0.1-1.diff.gz")
 
     def test_cmd_dailydeb_with_pristine_orig_gz_tarball(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         self.requireFeature(PristineTarFeature)
         self.make_simple_package("source")
         pristine_tar_sha1 = self.make_upstream_version("0.1",
@@ -497,12 +532,21 @@ class BlackboxBuilderTests(TestCaseWithTransport):
             pristine_tar_sha1)
 
     def test_cmd_dailydeb_with_pristine_orig_bz2_tarball(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         self.requireFeature(PristineTarFeature)
         self.make_simple_quilt_package()
-        pristine_tar_sha1 = self.make_upstream_version("0.1",
-            [("upstream/file", "content\n")], pristine_tar_format="bz2")
+        pristine_tar_sha1 = self.make_upstream_version("0.1", [
+            ("upstream/file", "content\n"),
+            ("upstream/a", "contents of source/a\n")],
+            pristine_tar_format="bz2")
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.3 "
                     "deb-version 0.1-1\nsource\n")])
+        wt = workingtree.WorkingTree.open("source")
+        self.build_tree([("source/upstream/")])
+        self.build_tree_contents([
+            ("source/upstream/file", "content\n"),
+            ("source/upstream/a", "contents of source/a\n")])
+        wt.add(["upstream", "upstream/file", "upstream/a"])
         out, err = self.run_bzr("dailydeb -q test.recipe working",
             retcode=0)
         self.assertPathExists("working/package_0.1.orig.tar.bz2")
@@ -511,7 +555,34 @@ class BlackboxBuilderTests(TestCaseWithTransport):
             osutils.sha_file_by_name("working/package_0.1.orig.tar.bz2"),
             pristine_tar_sha1)
 
+    def test_cmd_dailydeb_allow_fallback_to_native_with_orig_tarball(self):
+        self.requireFeature(PristineTarFeature)
+        self.make_simple_quilt_package()
+        pristine_tar_sha1 = self.make_upstream_version("0.1", [
+            ("upstream/file", "content\n"),
+            ("upstream/a", "contents of source/a\n")],
+            pristine_tar_format="bz2")
+        self.build_tree_contents([("test.recipe", "# bzr-builder format 0.3 "
+                    "deb-version 0.1-1\nsource\n")])
+        wt = workingtree.WorkingTree.open("source")
+        self.build_tree([("source/upstream/")])
+        self.build_tree_contents([
+            ("source/upstream/file", "content\n"),
+            ("source/upstream/a", "contents of source/a\n")])
+        wt.add(["upstream", "upstream/file", "upstream/a"])
+        out, err = self.run_bzr(
+            "dailydeb --allow-fallback-to-native -q test.recipe working",
+            retcode=0)
+        self.assertPathExists("working/package_0.1.orig.tar.bz2")
+        self.assertPathExists("working/package_0.1-1.debian.tar.gz")
+        self.assertEquals("3.0 (quilt)\n",
+            open("source/debian/source/format", "r").read())
+        self.assertEquals(
+            osutils.sha_file_by_name("working/package_0.1.orig.tar.bz2"),
+            pristine_tar_sha1)
+
     def test_cmd_dailydeb_force_native(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         self.make_simple_quilt_package()
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.3 "
                     "deb-version 1\nsource 2\n")])
@@ -522,6 +593,7 @@ class BlackboxBuilderTests(TestCaseWithTransport):
             "working/test-1/debian/source/format")
 
     def test_cmd_dailydeb_force_native_empty_series(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         source = self.make_simple_quilt_package()
         self.build_tree(['source/debian/patches/'])
         self.build_tree_contents([
@@ -538,6 +610,7 @@ class BlackboxBuilderTests(TestCaseWithTransport):
         self.assertPathDoesNotExist("working/test-1/debian/patches")
 
     def test_cmd_dailydeb_force_native_apply_quilt(self):
+        self.requireFeature(NotUnderFakeRootFeature)
         source = self.make_simple_quilt_package()
         self.build_tree(["source/debian/patches/"])
         patch = dedent(
@@ -602,7 +675,7 @@ class BlackboxBuilderTests(TestCaseWithTransport):
         source.commit("set source format")
 
         self.build_tree_contents([("test.recipe", "# bzr-builder format 0.3 "
-                    "deb-version 1\nsource\n")])
+                    "deb-version 1-1\nsource\n")])
         out, err = self.run_bzr(
             "dailydeb --allow-fallback-to-native -q test.recipe working", retcode=3)
         self.assertEquals(err, "bzr: ERROR: Unknown source format 2.0\n")
